@@ -2,6 +2,8 @@
 # Graphical User Interface for Biom3d
 # WARNING: with this current version the remote access mode only work with a
 # linux server.
+# WARNING: this script is only meant to work as main script and not as a
+# module. The imports are not included outside the __main__. 
 # Content:
 #  * Imports
 #  * Constants definition
@@ -21,27 +23,33 @@ from tkinter import filedialog
 import paramiko
 from stat import S_ISDIR, S_ISREG # for recursive download
 import os 
-
+import yaml
+from sys import platform 
+# if platform=='linux': # only import if linux because windows omero plugin requires Visual Studio Install which is too big
+import argparse
 
 from biom3d.configs.unet_default import CONFIG
 from biom3d.preprocess import preprocess
 from biom3d.auto_config import auto_config
-from biom3d.utils import nested_dict_change_value, save_config
 
 # the packages below are only needed for the local version of the GUI
+# WARNING! the lines below must be commented when deploying the remote version,
+# and uncommented when installing the local version.
 from biom3d.pred import pred
 from biom3d.builder import Builder
-
-from sys import platform
-# if platform=='linux': # only import if linux because windows omero plugin requires Visual Studio Install which is too big
 # import omero_pred
 
-
 #----------------------------------------------------------------------------
 # Constants 
 # remote or local
 
 REMOTE = False
+
+# The option below is made to remove the 'start locally' button in the gui. This is
+# useful for the deployment only in order to reduce the size of the 
+# distribution we only allow remote access. 
+LOCAL = False 
+
 MAIN_DIR = "/home/biome/biom3d" # folder containing biom3d repository on server computer
 TRANSPORT = False
 
@@ -56,9 +64,6 @@ ROOT_FRAME_STYLE = 'red_style.TFrame'
 NOTEBOOK_STYLE = 'red_style.TNotebook'
 NOTEBOOK_TAB_STYLE = 'red_style.TNotebook.Tab'
 
-# data tab
-# PATH_IMG_SELECT = 'images/select.png'
-
 #----------------------------------------------------------------------------
 # Styles 
 
@@ -205,6 +210,64 @@ class ParaProxy(paramiko.proxy.ProxyCommand):
         self.stderr.close()
         self.channel.close()        
 
+#----------------------------------------------------------------------------
+# general utils (also in biom3d.utils)
+
+class Dict(dict):
+    def __init__(self, *args, **kwargs): super().__init__(*args, **kwargs)
+    def __getattr__(self, name): return self[name]
+    def __setattr__(self, name, value): self[name] = value
+    def __delattr__(self, name): del self[name]
+
+def Dict_to_dict(cfg):
+    """
+    transform a Dict into a dict
+    """
+    ty = type(cfg)
+    cfg = dict(cfg)
+    for k,i in cfg.items():
+        if type(i)==ty:
+            cfg[k] = Dict_to_dict(cfg[k])
+    return cfg
+
+def save_config(path, cfg):
+    """
+    save a configuration in a yaml file.
+    path must thus contains a yaml extension.
+    example: path='logs/test.yaml'
+    """
+    cfg = Dict_to_dict(cfg)
+    with open(path, "w") as f:
+        yaml.dump(cfg, f, sort_keys=False)
+
+def nested_dict_pairs_iterator(dic):
+    ''' This function accepts a nested dictionary as argument
+        and iterate over all values of nested dictionaries
+        stolen from: https://thispointer.com/python-how-to-iterate-over-nested-dictionary-dict-of-dicts/ 
+    '''
+    # Iterate over all key-value pairs of dict argument
+    for key, value in dic.items():
+        # Check if value is of dict type
+        if isinstance(value, dict) or isinstance(value, Dict):
+            # If value is dict then iterate over all its values
+            for pair in  nested_dict_pairs_iterator(value):
+                yield [key, *pair]
+        else:
+            # If value is not dict type then yield the value
+            yield [key, value]
+
+def nested_dict_change_value(dic, key, value):
+    """
+    Change all value with a given key from a nested dictionary.
+    """
+    # Loop through all key-value pairs of a nested dictionary and change the value 
+    for pairs in nested_dict_pairs_iterator(dic):
+        if key in pairs:
+            save = dic[pairs[0]]; i=1
+            while i < len(pairs) and pairs[i]!=key:
+                save = save[pairs[i]]; i+=1
+            save[key] = value
+    return dic
 
 #----------------------------------------------------------------------------
 # File dialog
@@ -713,7 +776,7 @@ class Connect2Omero(ttk.LabelFrame):
         self.username_entry = ttk.Entry(self, textvariable=self.username)
 
         self.password_label = ttk.Label(self, text='Password:')
-        self.password = StringVar(value="biome")
+        self.password = StringVar(value="")
         self.password_entry = ttk.Entry(self, textvariable=self.password, show='*')
 
         # place widgets
@@ -765,9 +828,15 @@ class ModelSelection(ttk.LabelFrame):
 
             # define the dropdown menu
             self.logs_dir = StringVar(value=self.logs_list[0])
-            self.logs_dir_option_menu = ttk.OptionMenu(self, self.logs_dir, self.logs_list[0], *self.logs_list, command=self._update_logs_list)
+            self.logs_dir_option_menu = ttk.OptionMenu(self, self.logs_dir, self.logs_list[0], *self.logs_list)
+            self.button_update_list = ttk.Button(self, text="Update", command=self._update_logs_list)
 
             self.logs_dir_option_menu.grid(column=0, row=0, sticky=(W,E))
+            self.button_update_list.grid(column=1, row=0, sticky=(W,E))
+
+            self.columnconfigure(0, weight=10)
+            self.columnconfigure(1, weight=1)
+            self.rowconfigure(0, weight=1)
 
         else: 
             ## build folder
@@ -776,13 +845,13 @@ class ModelSelection(ttk.LabelFrame):
 
             self.logs_dir.grid(column=0, row=0, sticky=(W,E))
 
-        self.columnconfigure(0, weight=1)
-        self.rowconfigure(0, weight=1)
+            self.columnconfigure(0, weight=1)
+            self.rowconfigure(0, weight=1)
         
-    def _update_logs_list(self, value):
+    def _update_logs_list(self):
         _,stdout,_ = REMOTE.exec_command('ls {}/logs'.format(MAIN_DIR))
-        self.logs_dir = [e.replace('\n','') for e in stdout.readlines()]
-        self.logs_dir_option_menu.set_menu(self.logs_dir[0], *self.logs_dir)
+        self.logs_list = [e.replace('\n','') for e in stdout.readlines()]
+        self.logs_dir_option_menu.set_menu(self.logs_list[0], *self.logs_list)
 
 class OutputDirectory(ttk.LabelFrame):
     def __init__(self, *arg, **kw):
@@ -826,20 +895,23 @@ class DownloadPrediction(ttk.LabelFrame):
         _,stdout,_ = REMOTE.exec_command('ls {}/data/pred'.format(MAIN_DIR))
         self.data_list = [e.replace('\n','') for e in stdout.readlines()]
         self.data_dir = StringVar(value=self.data_list[0])
-        self.data_dir_option_menu = ttk.OptionMenu(self, self.data_dir, self.data_list[0], *self.data_list, command=self._update_pred_list)
+        self.data_dir_option_menu = ttk.OptionMenu(self, self.data_dir, self.data_list[0], *self.data_list)
+        self.button_update_list = ttk.Button(self, text="Update", command=self._update_pred_list)
 
         # or send the dataset to server
         self.get_data_label = ttk.Label(self, text="Select local folder to download into:")
         self.get_data_folder = FileDialog(self, mode='folder', textEntry="data/pred")
         self.get_data_button = ttk.Button(self, text="Get data", command=self.get_data)
 
-        self.input_folder_label.grid(column=0, row=0, sticky=(W,E))
+        self.input_folder_label.grid(column=0, row=0, columnspan=2, sticky=(W,E))
         self.data_dir_option_menu.grid(column=0, row=1, sticky=(W,E))
-        self.get_data_label.grid(column=0, row=2, sticky=(W,E))
-        self.get_data_folder.grid(column=0, row=3, sticky=(W,E))
-        self.get_data_button.grid(column=0, row=4, sticky=(W,E))
+        self.button_update_list.grid(column=1, row=1, sticky=(W,E))
+        self.get_data_label.grid(column=0, row=2, columnspan=2, sticky=(W,E))
+        self.get_data_folder.grid(column=0, row=3, columnspan=2, sticky=(W,E))
+        self.get_data_button.grid(column=0, row=4, columnspan=2, sticky=(W,E))
 
-        self.columnconfigure(0, weight=1)
+        self.columnconfigure(0, weight=10)
+        self.columnconfigure(1, weight=1)
         for i in range(5):
             self.rowconfigure(i, weight=1)
     
@@ -860,7 +932,7 @@ class DownloadPrediction(ttk.LabelFrame):
         # copy files from remote to local
         ftp_get_folder(ftp, remotedir, localdir)
     
-    def _update_pred_list(self, value=None):
+    def _update_pred_list(self):
         _,stdout,_ = REMOTE.exec_command('ls {}/data/pred'.format(MAIN_DIR))
         self.data_list = [e.replace('\n','') for e in stdout.readlines()]
         self.data_dir_option_menu.set_menu(self.data_list[0], *self.data_list)
@@ -995,7 +1067,7 @@ class Connect2Remote(ttk.LabelFrame):
 
         # widgets definitions
         self.hostname_label = ttk.Label(self, text='Server address:')
-        self.hostname = StringVar(value="MPCRBC-GRED-077")
+        self.hostname = StringVar(value="")
         self.hostname_entry = ttk.Entry(self, textvariable=self.hostname)
 
         self.username_label = ttk.Label(self, text='User name:')
@@ -1003,7 +1075,7 @@ class Connect2Remote(ttk.LabelFrame):
         self.username_entry = ttk.Entry(self, textvariable=self.username)
 
         self.password_label = ttk.Label(self, text='Password:')
-        self.password = StringVar(value="biome")
+        self.password = StringVar(value="")
         self.password_entry = ttk.Entry(self, textvariable=self.password, show='*')
 
         self.main_dir_label = ttk.Label(self, text='Folder of Biom3d repository on remote server:')
@@ -1042,11 +1114,11 @@ class Connect2Remote(ttk.LabelFrame):
         """
         if self.use_proxy_state.get(): # place widgets
             self.proxy_hostname_label = ttk.Label(self, text='Proxy server address:')
-            self.proxy_hostname = StringVar(value="bowser.igred.fr")
+            self.proxy_hostname = StringVar(value="")
             self.proxy_hostname_entry = ttk.Entry(self, textvariable=self.proxy_hostname)
 
             self.proxy_username_label = ttk.Label(self, text='Proxy user name:')
-            self.proxy_username = StringVar(value="mougeotg")
+            self.proxy_username = StringVar(value="")
             self.proxy_username_entry = ttk.Entry(self, textvariable=self.proxy_username)
 
             self.proxy_password_label = ttk.Label(self, text='Proxy password:')
@@ -1123,6 +1195,7 @@ class Root(Tk):
 
         self.title_label = ttk.Label(self.local_or_remote, text="Biom3d", font=("Montserrat", 18))
         self.welcome_message = ttk.Label(self.local_or_remote, text="Welcome!\n\nBiom3d is an easy-to-use tool to train and use deep learning models for segmenting three dimensional images. You can either start locally, if your computer has a good graphic card (NVIDIA Geforce RTX 1080 or higher) or connect remotelly on a computer with such a graphic card.\n\nIf you need help, check our GitHub repository here: https://github.com/GuillaumeMougeot/biom3d", anchor="w", justify=LEFT, wraplength=450)
+
         self.start_locally = ttk.Button(self.local_or_remote, text="Start locally", command=lambda: self.main(remote=False))
 
         self.start_remotelly_frame = Connect2Remote(self.local_or_remote, text="Connect to remote server", padding=[10,10,10,10])
@@ -1130,7 +1203,10 @@ class Root(Tk):
 
         self.title_label.grid(column=0, row=0, sticky=W)
         self.welcome_message.grid(column=0, row=1, sticky=(W,E), pady=12)
-        self.start_locally.grid(column=0, row=2, sticky=(W,E), pady=12)
+        
+        # The local button is displayed only for the local installation 
+        if LOCAL: 
+            self.start_locally.grid(column=0, row=2, sticky=(W,E), pady=12)
 
         self.start_remotelly_frame.grid(column=0, row=3, sticky=(W,E), pady=12)
         self.start_remotelly_button.grid(column=0, row=4, sticky=(W,E), pady=5)
@@ -1224,12 +1300,21 @@ class Root(Tk):
         self.train_tab.columnconfigure(0, weight=1)
         self.train_tab.rowconfigure(0, weight=1)
 
-root = Root()
+if __name__=='__main__':
+    parser = argparse.ArgumentParser(description="Graphical User Interface of Biom3d")
+    parser.add_argument("-L", "--local", default=True,  action='store_true', dest='local',
+        help="Start the GUI with the local version (the remote version is the default version).") 
+    args = parser.parse_args()
 
+    LOCAL = args.local
 
-try: # avoid blury UI on Windows
-    if platform=='win32':
-        from ctypes import windll
-        windll.shcore.SetProcessDpiAwareness(1)
-finally:
-    root.mainloop()
+    root = Root()
+
+    try: # avoid blury UI on Windows
+        if platform=='win32':
+            from ctypes import windll
+            windll.shcore.SetProcessDpiAwareness(1)
+    finally:
+        root.mainloop()
+
+    
