@@ -4,26 +4,23 @@
 # TODO: re-structure with classes maybe?
 #---------------------------------------------------------------------------
 
-from cgitb import enable
 import torch 
 import torchio as tio
 import numpy as np
 from skimage.io import imread
 from tqdm import tqdm
-from scipy.ndimage.filters import gaussian_filter
+# from scipy.ndimage.filters import gaussian_filter
 
-from biom3d.utils import keep_biggest_volume_centered, sitk_imread
+from biom3d.utils import keep_biggest_volume_centered, adaptive_imread
 
 #---------------------------------------------------------------------------
 # model predictor for segmentation
 
 def load_img_seg(fname):
     img = imread(fname)
-    # img = utils.sitk_imread()(fname)
 
     # normalize
     img = (img - img.min()) / (img.max() - img.min())
-    # img = (img-img.mean())/img.std()
     
     # to tensor
     img = torch.tensor(img, dtype=torch.float)
@@ -88,8 +85,7 @@ class LoadImgPatch:
         
         # prepare image
         # load the image
-        # img = imread(fname)
-        img,self.spacing = sitk_imread(self.fname)
+        img,self.spacing = adaptive_imread(self.fname)
 
         # store img shape (for post processing)
         self.img_shape = img.shape
@@ -161,30 +157,6 @@ class LoadImgPatch:
         else:
             return logit
 
-def load_img_seg_patch(fname, patch_size=(64,64,32)):
-    """
-    Prepare image for model prediction
-    """
-    # load the image
-    # img = imread(fname)
-    img,_ = sitk_imread(fname)
-
-    # normalize the image
-    # bits = lambda x: ((np.log2(x)>8).astype(int)+(np.log2(x)>16).astype(int)*2+1)*8
-    # img = img / (2**bits(np.max(img)) - 1)
-    img = (img-img.mean())/img.std()
-    img = np.expand_dims(img, 0)
-    img = torch.from_numpy(img).float()
-
-    # define the grid sampler 
-    sub = tio.Subject(img=tio.ScalarImage(tensor=img))
-    patch_size = np.array(patch_size)
-    sampler= tio.data.GridSampler(subject=sub, 
-                            patch_size=patch_size, 
-                            patch_overlap=patch_size//2,
-                            padding_mode='constant')
-    return sampler
-
 def seg_predict_patch(
     img_path,
     model,
@@ -196,7 +168,7 @@ def seg_predict_patch(
     intensity_moments=[],
     use_softmax=False,
     num_workers=4,
-    # enable_autocast=True, 
+    enable_autocast=True, 
     keep_biggest_only=False,
     ):
     """
@@ -204,9 +176,9 @@ def seg_predict_patch(
     """
 
     device = 'cuda' if torch.cuda.is_available() else 'cpu'
-    enable_autocast = torch.cuda.is_available() # tmp, autocast seems to work only with gpu for now... 
+    enable_autocast = torch.cuda.is_available() and enable_autocast # tmp, autocast seems to work only with gpu for now... 
+    print('AMP {}'.format('enable' if enable_autocast else 'disable'))
 
-    # img = load_img_seg_patch(img_path, patch_size)
     img_loader = LoadImgPatch(
         img_path,
         patch_size,
@@ -234,62 +206,54 @@ def seg_predict_patch(
                 X = X.cuda()
             
             if tta: # test time augmentation: flip around each axis
-                # Xs = [X] + [torch.flip(X,dims=[i]) for i in range(2,5)]
-                # with torch.cuda.amp.autocast():
-                #     preds = [model(x).cpu() for x in Xs]
-                # preds = [preds[0]]+[torch.flip(preds[i], dims=[i+1]) for i in range(1,4)]
-                # pred = torch.mean(torch.stack(preds), dim=0)
-
                 with torch.autocast(device, enabled=enable_autocast):
                     pred=model(X).cpu()
-                    # pred+=torch.flip(pred, dims=[1]).cpu()
                 
                 # flipping tta
-                # dims = [[1],[2],[3],[1,2],[1,3],[2,3],[1,2,3]]
                 dims = [[2],[3],[4]]
-                # dims = [[2]]
                 for i in range(len(dims)):
                     X_flip = torch.flip(X,dims=dims[i])
 
-                    # with torch.cuda.amp.autocast():
                     with torch.autocast(device, enabled=enable_autocast):
                         pred_flip = model(X_flip)
                         pred += torch.flip(pred_flip, dims=dims[i]).cpu()
                     
                     del X_flip, pred_flip
-                    # torch.cuda.empty_cache()
                 
                 pred = pred/(len(dims)+1)
-                # pred = pred/3
                 del X
-                # torch.cuda.empty_cache()
             else:
-                # with torch.cuda.amp.autocast():
                 with torch.autocast(device, enabled=enable_autocast):
                     pred=model(X).cpu()
                     del X
-                    # torch.cuda.empty_cache()
+
             pred_aggr.add_batch(pred, patch[tio.LOCATION])
         
+        print("Prediction done!")
+
+        print("Aggregation...")
         logit = pred_aggr.get_output_tensor().float()
+        print("Aggregation done!")
     
     if torch.cuda.is_available():
         torch.cuda.empty_cache()
 
     # post-processing:
+    print("Post-processing...")
     logit = img_loader.post_process(logit)
 
-    if return_logit: return logit.numpy()
+    if return_logit: 
+        print("Post-processing done!")
+        return logit.numpy()
 
     if use_softmax:
-        # out = (logit.softmax(dim=0)>0.5).int()[1:]
         out = (logit.softmax(dim=0).argmax(dim=0)).int()
     else:
         out = (logit.sigmoid()>0.5).int()
     out = out.numpy()
 
+    # TODO: the function below is too slow
     if keep_biggest_only:
-        # out = utils.keep_center_only(out)
         if len(out.shape)==3:
             out = keep_biggest_volume_centered(out)
         elif len(out.shape)==4:
@@ -299,7 +263,8 @@ def seg_predict_patch(
             out = np.array(tmp)
 
     out = out.astype(np.byte) 
-    print("output shape",out.shape)
+    print("Post-processing done!")
+    print("Output shape:",out.shape)
     return out
 
 #---------------------------------------------------------------------------
